from ScriptResult import EXECUTION_STATE_COMPLETED, EXECUTION_STATE_FAILED
from SiemplifyAction import SiemplifyAction
from SiemplifyUtils import output_handler
from SlackManager import SlackManager, SlackManagerException
from TIPCommon import extract_configuration_param, extract_action_param
from consts import PROVIDER_NAME

SCRIPT_NAME = f'{PROVIDER_NAME} - GetUserDetailsById'


@output_handler
def main():
    siemplify = SiemplifyAction()
    siemplify.script_name = SCRIPT_NAME
    siemplify.LOGGER.info('----------------- Main - Param Init -----------------')

    api_token = extract_configuration_param(
        siemplify,
        provider_name=PROVIDER_NAME,
        param_name='ApiToken',
        input_type=str
    )

    user_id = extract_action_param(
        siemplify,
        param_name='User Id',
        is_mandatory=True,
        print_value=True,
        input_type=str
    )

    siemplify.LOGGER.info('----------------- Main - Started -----------------')

    try:
        slack_manager = SlackManager(api_token)
        user = slack_manager.get_user_details_by_id(user_id)
        output_message = f'User with id {user_id} has been received.'
        result = 'true'
        status = EXECUTION_STATE_COMPLETED
        siemplify.LOGGER.info(f'Script Name: {SCRIPT_NAME} | {output_message}')
        siemplify.result.add_result_json(user.raw_data)
    except (SlackManagerException, Exception) as e:
        output_message = f'An error occurred when trying to receive user with id {user_id}: {e}'
        result = 'false'
        status = EXECUTION_STATE_FAILED
        siemplify.LOGGER.error(f'Script Name: {SCRIPT_NAME} | {output_message}')
        siemplify.LOGGER.exception(e)

    siemplify.LOGGER.info('----------------- Main - Finished -----------------')
    siemplify.LOGGER.info(f'Status: {status}')
    siemplify.LOGGER.info(f'Result: {result}')
    siemplify.LOGGER.info(f'Output Message: {output_message}')

    siemplify.end(output_message, result, status)


if __name__ == '__main__':
    main()
